<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 11/26/2009 *
 * ===================                          Last change: 11/26/2009 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : what-list_network_params.php                     *
 * -------------------------------------------------------------------- *
 * Short description : List/manage network API query parameters         *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Verwaltet API-Abfrageparameter                   *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * Needs to be in all Files and every File needs "svn propset           *
 * svn:keywords Date Revision" (autoprobset!) at least!!!!!!            *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009, 2010 by Mailer Developer Team                    *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if ((!defined('__SECURITY')) || (!isAdmin())) {
	die();
} // END - if

// Add description as navigation point
addMenuDescription('admin', __FILE__);

// These are expert settings!
if (doVerifyExpertSettings() != 'agreed') {
	// Abort here
	return;
} // END - if

// By default we should display list/add new forms
$GLOBALS['network_display'] = true;

// Handle form here
doNetworkHandleForm();

// Display forms?
if ($GLOBALS['network_display'] === false) {
	// Abort here
	return;
} // END - if

// Do we have a network selected?
if (isGetRequestParameterSet('network')) {
	// Get its data for template
	$networkData = getNetworkDataById(getRequestParameter('network'));

	// Init disabled list
	$networkData['disabled'] = array();

	// Only show params from given type handler if choosen
	$ADD = '';
	if (isPostRequestParameterSet('network_type_id')) {
		// Only show from current type handler
		$ADD = sprintf(" AND p.`network_type_id`=%s", bigintval(postRequestParameter('network_type_id')));

		// Display message
		loadTemplate('admin_settings_saved', false, getMessage('ADMIN_NETWORK_PARAMS_LIST_LIMITED_TO_TYPE'));
	} // END - if

	// Load all advert types for given network
	$result = SQL_QUERY_ESC("SELECT
	p.`network_param_id`, t.`network_type_id`, t.`network_type_handle`, p.`request_param_key`, p.`request_param_value`, p.`request_param_default`
FROM
	`{?_MYSQL_PREFIX?}_network_request_params` AS p
INNER JOIN
	`{?_MYSQL_PREFIX?}_network_types` AS t
ON
	p.`network_type_id`=t.`network_type_id`
WHERE
	p.`network_id`=%s".$ADD."
ORDER BY
	t.`network_type_handle` ASC,
	p.`network_param_id` ASC",
		array(getRequestParameter('network')), __FILE__, __LINE__);

	// Do we have entries?
	if (SQL_NUMROWS($result) > 0) {
		// List all entries
		$OUT = ''; $SW = 2;
		while ($row = SQL_FETCHARRAY($result)) {
			// Add/translate some data
			$row['sw'] = $SW;

			// Add it to network if the type matches selected
			if ($row['network_type_id'] == postRequestParameter('network_type_id')) {
				// Matches, then add it for disabling this entry
				$networkData['disabled'][$row['request_param_key']] = true;
			} // END - if

			// Is default value empty or null?
			if ((is_null($row['request_param_default'])) || (trim($row['request_param_default']) == '')) {
				// Fix it to three dashes
				$row['request_param_default'] = '---';
			} // END - if

			// Add row template and switch color
			$OUT .= loadTemplate('admin_list_network_params_row', true, $row);
			$SW = 3 - $SW;
		} // END - while

		// Prepare data for template
		$content = array(
			'rows'            => $OUT,
			'network_id'      => getRequestParameter('network'),
			'network_title'   => $networkData['network_title'],
			'network_reflink' => $networkData['network_reflink'],
		);

		// Load main template
		loadTemplate('admin_list_network_params', false, $content);
	} else {
		// No entries found
		loadTemplate('admin_settings_saved', false, getMaskedMessage('ADMIN_NETWORK_PARAMS_404', $networkData['network_title']));
	}

	// Free result
	SQL_FREERESULT($result);

	// Add options list for network type
	$networkData['type_options'] = generateOptionList(
		'network_types',
		'network_type_id',
		'network_type_handle',
		postRequestParameter('network_type_id'),
		'',
		sprintf("WHERE `network_id`=%s", getRequestParameter('network'))
	);

	// Add options for parameter keys
	$networkData['key_options'] = generateOptionList(
		'/ARRAY/',
		array(
			'id',
			'sid',
			'hash',
			'password',
			'reload',
			'max_stay',
			'min_stay',
			'currency',
			'type',
			'remain',
			'reward',
			'size',
			'erotic',
			'extra'
		),
		array(
			'{--ADMIN_NETWORK_PARAMS_ID--}',
			'{--ADMIN_NETWORK_PARAMS_SID--}',
			'{--ADMIN_NETWORK_PARAMS_HASH--}',
			'{--ADMIN_NETWORK_PARAMS_PASSWORD--}',
			'{--ADMIN_NETWORK_PARAMS_RELOAD--}',
			'{--ADMIN_NETWORK_PARAMS_MAXIMUM_STAY--}',
			'{--ADMIN_NETWORK_PARAMS_MINIMUM_STAY--}',
			'{--ADMIN_NETWORK_PARAMS_CURRENCY--}',
			'{--ADMIN_NETWORK_PARAMS_TYPE--}',
			'{--ADMIN_NETWORK_PARAMS_REMAIN--}',
			'{--ADMIN_NETWORK_PARAMS_REWARD--}',
			'{--ADMIN_NETWORK_PARAMS_SIZE--}',
			'{--ADMIN_NETWORK_PARAMS_EROTIC--}',
			'{--ADMIN_NETWORK_PARAMS_EXTRA--}'
		),
		'', '', '',
		$networkData['disabled']
	);

	// Add form for adding new params
	loadTemplate('admin_add_network_params', false, $networkData);
} else {
	// Generate network list for this script
	outputHtml(generateAdminNetworkList());
}

// [EOF]
?>
